@@ -1,9 +1,5 @@
-#!/usr/bin/env python
-
+#!/usr/bin/env python3
 """Command-line interface for CNVkit, the Copy Number Variation toolkit."""
-from future import standard_library
-standard_library.install_aliases()
-
 import logging
 from cnvlib import commands
 
