@@ -82,23 +82,29 @@ def load_adjust_coverages(cnarr, ref_cnarr, skip_low,
         logging.warning("WARNING: most bins have no or very low coverage; "
                         "check that the right BED file was used")
     else:
+        cnarr_index_reset = False
         if fix_gc:
             if 'gc' in ref_matched:
                 logging.info("Correcting for GC bias...")
                 cnarr = center_by_window(cnarr, .1, ref_matched['gc'])
+                cnarr_index_reset = True
             else:
                 logging.warning("WARNING: Skipping correction for GC bias")
         if fix_edge:
             logging.info("Correcting for density bias...")
             edge_bias = get_edge_bias(cnarr, params.INSERT_SIZE)
             cnarr = center_by_window(cnarr, .1, edge_bias)
+            cnarr_index_reset = True
         if fix_rmask:
             if 'rmask' in ref_matched:
                 logging.info("Correcting for RepeatMasker bias...")
                 cnarr = center_by_window(cnarr, .1, ref_matched['rmask'])
+                cnarr_index_reset = True
             else:
                 logging.warning("WARNING: Skipping correction for "
                                 "RepeatMasker bias")
+        if cnarr_index_reset:
+            ref_matched.data.reset_index(drop=True, inplace=True)
     return cnarr, ref_matched
 
 
