@@ -1,8 +1,4 @@
 """Base class for an array of annotated genomic regions."""
-from __future__ import print_function, absolute_import, division
-from builtins import next, object, zip
-from past.builtins import basestring
-
 import logging
 from collections import OrderedDict
 
@@ -50,9 +46,12 @@ class GenomicArray(object):
             else:
                 def ok_dtype(col, dt):
                     return data_table[col].dtype == np.dtype(dt)
-            for col, dtype in zip(self._required_columns, self._required_dtypes):
-                if not ok_dtype(col, dtype):
-                    data_table[col] = data_table[col].astype(dtype)
+            recast_cols = {col: dtype
+                for col, dtype in zip(self._required_columns, self._required_dtypes)
+                if not ok_dtype(col, dtype)
+            }
+            if recast_cols:
+                data_table = data_table.astype(recast_cols)
 
         self.data = data_table
         self.meta = (dict(meta_dict)
@@ -90,17 +89,14 @@ class GenomicArray(object):
         return self.__class__.from_columns(columns, self.meta)
         # return self.__class__(self.data.loc[:, columns], self.meta.copy())
 
-    def as_dataframe(self, dframe):
+    def as_dataframe(self, dframe, reset_index=False):
         """Wrap the given pandas DataFrame in this instance's metadata."""
-        return self.__class__(dframe.reset_index(drop=True), self.meta.copy())
+        if reset_index:
+            dframe = dframe.reset_index(drop=True)
+        return self.__class__(dframe, self.meta.copy())
 
-    # def as_index(self, index):
-    #     """Subset with fancy/boolean indexing; reuse this instance's metadata."""
-    #     """Extract rows by indices, reusing this instance's metadata."""
-    #     if isinstance(index, (int, slice)):
-    #         return self.__class__(self.data.iloc[index], self.meta.copy())
-    #     else:
-    #         return self.__class__(self.data[index], self.meta.copy())
+    def as_series(self, arraylike):
+        return pd.Series(arraylike, index=self.data.index)
 
     def as_rows(self, rows):
         """Wrap the given rows in this instance's metadata."""
@@ -121,8 +117,6 @@ class GenomicArray(object):
     def __bool__(self):
         return bool(len(self.data))
 
-    __nonzero__ = __bool__  # Py2.7
-
     def __eq__(self, other):
         return (isinstance(other, self.__class__) and
                 self.data.equals(other.data))
@@ -147,7 +141,7 @@ class GenomicArray(object):
             # A single row
             return self.data.iloc[index]
             # return self.as_dataframe(self.data.iloc[index:index+1])
-        elif isinstance(index, basestring):
+        elif isinstance(index, str):
             # A column, by name
             return self.data[index]
         elif (isinstance(index, tuple) and
@@ -175,7 +169,7 @@ class GenomicArray(object):
         """Assign to a portion of the data."""
         if isinstance(index, int):
             self.data.iloc[index] = value
-        elif isinstance(index, basestring):
+        elif isinstance(index, str):
             self.data[index] = value
         elif (isinstance(index, tuple) and
               len(index) == 2 and
@@ -218,7 +212,7 @@ class GenomicArray(object):
             # The autosomes, if any, are not named with plain integers
             return self
         if also:
-            if isinstance(also, basestring):
+            if isinstance(also, str):
                 also = [also]
             for a_chrom in also:
                 is_auto |= (self.chromosome == a_chrom)
@@ -315,7 +309,7 @@ class GenomicArray(object):
         """
         cols = list(GenomicArray._required_columns)
         if also:
-            if isinstance(also, basestring):
+            if isinstance(also, str):
                 cols.append(also)
             else:
                 cols.extend(also)
@@ -387,7 +381,8 @@ class GenomicArray(object):
             Concatenation of all the subsets of `self` enclosed by the specified
             ranges.
         """
-        table = pd.concat(iter_ranges(self.data, chrom, starts, ends, mode))
+        table = pd.concat(iter_ranges(self.data, chrom, starts, ends, mode),
+                          sort=False)
         return self.as_dataframe(table)
 
     def into_ranges(self, other, column, default, summary_func=None):
@@ -519,11 +514,7 @@ class GenomicArray(object):
             A new instance of `self` with the given columns included in the
             underlying dataframe.
         """
-        # return self.as_dataframe(self.data.assign(**columns))
-        result = self.copy()
-        for key, values in columns.items():
-            result[key] = values
-        return result
+        return self.as_dataframe(self.data.assign(**columns))
 
     def keep_columns(self, colnames):
         """Extract a subset of columns, reusing this instance's metadata."""
