@@ -1,11 +1,13 @@
 #!/usr/bin/env python
 """Unit tests for the CNVkit library, cnvlib."""
-from __future__ import absolute_import, division, print_function
-
 import unittest
 
-from skgenome import tabio
+# unittest/pomegranate 0.10.0: ImportWarning: can't resolve package from
+# __spec__ or __package__, falling back on __name__ and __path__
+import warnings
+warnings.filterwarnings('ignore', category=ImportWarning)
 
+from skgenome import tabio
 
 class IOTests(unittest.TestCase):
     """Tests for I/O modules."""
