@@ -1,8 +1,13 @@
 #!/usr/bin/env python
 """Unit tests for CNVkit that require an R installation."""
-from __future__ import absolute_import, division, print_function
-
 import unittest
+import logging
+logging.basicConfig(level=logging.ERROR, format="%(message)s")
+
+# unittest/pomegranate 0.10.0: ImportWarning: can't resolve package from
+# __spec__ or __package__, falling back on __name__ and __path__
+import warnings
+warnings.filterwarnings('ignore', category=ImportWarning)
 
 import cnvlib
 from cnvlib import segmentation
